@@ -1,8 +1,15 @@
-use evercrypt::prelude::random_vec;
+use rand::{rngs::OsRng, RngCore};
 
 #[inline]
 pub(crate) fn random(l: usize) -> Vec<u8> {
-    random_vec(l)
+    // TODO: This function must be replaced with a proper PRNG call.
+    debug_assert!(
+        l < 128,
+        "We only want to take a little bit of randomness here."
+    );
+    let mut out = vec![0u8; l];
+    OsRng.fill_bytes(&mut out);
+    out
 }
 
 #[inline]
